package MT::Plugin::SKR::FixThemeExport;
#  fix template identifiers in the blogs/websites
# for exporting theme
#
#       Copyright (c) 2011 SKYARC System Co.,Ltd.

use strict;
use MT;
use MT::Blog;
use MT::I18N;
#use Data::Dumper;#DEBUG

use MT::Template;

use vars qw( $MYNAME $VERSION );
$MYNAME = 'FixThemeExport';
$VERSION = '1.0';

use base qw( MT::Plugin );
my $plugin = __PACKAGE__->new({
        name => $MYNAME,
        id => lc $MYNAME,
        key => lc $MYNAME,
        version => $VERSION,
        author_name => 'SKYARC System Co.,Ltd.',
        author_link => 'http://www.skyarc.co.jp/',
        doc_link => 'http://www.skyarc.co.jp/',
        description => <<HTMLHEREDOC,
<__trans phrase="Fix template identifiers in the blogs/websites for exporting theme.">
HTMLHEREDOC
        l10n_class  => $MYNAME. '::L10N',
});
MT->add_plugin( $plugin );

sub instance { $plugin; }

### Registry
sub init_registry {
    my $plugin = shift;
    $plugin->registry({
		applications => {
			'cms' => {
				methods => {
					fix_theme_export => '$FixThemeExport::MT::Plugin::SKR::FixThemeExport::_hdlr_fix_theme_export',
				},
			},
		},
		callbacks => {
			'MT::App::CMS::template_source.export_theme' =>
				'$FixThemeExport::MT::Plugin::SKR::FixThemeExport::_hdlr_export_theme_source',
			'MT::App::CMS::template_param.export_theme' =>
				'$FixThemeExport::MT::Plugin::SKR::FixThemeExport::_hdlr_export_theme_param',
		},
    });
}

sub _check_permission {
    my $app  = MT->instance;
	my $blog = $app->blog;

	my $author = $app->user or return 0;

	return 1 if $author->is_superuser;
	my $perm = $author->permissions(0);
	return 1 if $perm->can_do('administer');

	if ($blog) {

         $perm = $author->permissions( $blog->id ) or return 0;
#		 MT->log('user : ' . $author->name . ' trace' );
		 return $blog->is_blog
			 ? $perm->can_do('administer_blog')
			 : $perm->can_do('administer_website');
	}

	return 0;
}

# handler for 'export_theme' template source.
sub _hdlr_export_theme_source {
	my ($cb, $app, $tmpl_ref) = @_;

	return unless _check_permission();

	my $plugin = MT->component($MYNAME);

	my $msg =<<__HTML__;
<mt:if name=fix_template>
    <div id="msg-block">
    <mt:if name="fix_template">
        <mtapp:statusmsg
            id="fix_template-theme"
            class="success">
            <__trans phrase="Templates have been fixed for export.">
        </mtapp:statusmsg>
    </mt:if>
    </div>
</mt:if>
__HTML__
	$msg = $plugin->translate_templatized($msg);

	$$tmpl_ref =~ s/(<mt:setvarblock name="system_msg">)/$1$msg/;

	my $start = quotemeta('<mt:setvarblock name="action_buttons">');
	my $middle = quotemeta('</mt:setvarblock>');
	my $end = quotemeta('</form>');

	my $new_head = "<mt:unless name=needs_fix_template>";
	my $new_middle= "</mt:unless>";
	my $new_tail =<<__HTML__;
<mt:if name=needs_fix_template>
<form method="post" enctype="multipart/form-data" action="<mt:var name="script_url">">
<input type="hidden" name="__mode" value="fix_theme_export" />
<input type="hidden" name="blog_id" value="<mt:var name="blog_id">" />
<input type="hidden" name="magic_token" value="<mt:var name="magic_token">" />
<input type="hidden" name="theme_class" value="<mt:var name="theme_class">" />
<button
    type="submit"
    accesskey="s"
    title="<MT_TRANS phrase="Fix Theme For Export">"
    class="export action primary-button">
    <MT_TRANS phrase="Fix Theme For Export"></button>
</form>
</mt:if>
__HTML__
	$new_tail = $plugin->translate_templatized($new_tail);

	my $status = $$tmpl_ref =~ s/($start.+?$middle)(.+?$end)/$new_head$1$new_middle$2$new_tail/ms;
}

# handler for 'export_theme' params.
sub _hdlr_export_theme_param {
    my ($cb, $app, $param, $tmpl) = @_;

	return unless _check_permission();

	my $plugin = MT->component($MYNAME);

	my $blog = $app->blog
		or return $cb->error($plugin->translate('No blog'));

	my $blog_id = $blog->id;

	my $terms = {
		blog_id => $blog_id,
		identifier => { like => '%/%' },
	};

	# Is there template which can't export ?
	my $count = MT::Template->count($terms);
	if ( $count > 0 ) {
		$param->{needs_fix_template} = 1;
	}

	# set flag for fixing template
	if ( $app->param('fix_template') ) {
		$param->{fix_template} = 1;
	}
}

# action handler for fixing template identifers
sub _hdlr_fix_theme_export {
	my $app = MT->instance;
	my $plugin = MT->component($MYNAME);

	_check_permission
		or return $app->error($plugin->translate('Permission Error.'));

	# check magic
	return unless $app->validate_magic();

	my $blog = $app->blog
		or return $app->error($plugin->translate('No blog'));

	my $blog_id = $blog->id;

	# find templates which has '/' in their own identifer.
	my $terms = {
		blog_id => $blog_id,
		identifier => { like => '%/%' },
	};

	my $iter = MT::Template->load_iter($terms);

	while (my $template = $iter->()) {
		# replace '/' with '_' in identifier.
		my $identifier = $template->identifier;

		$identifier =~ s!/!_!g;

		$template->identifier($identifier);

		$template->save
			or return $app->error($plugin->translate('Faild to save template [_1]', $template->id));
	}

	return $app->redirect($app->uri(
		mode => 'export_theme',
		args => {
			blog_id => $blog_id,
			fix_template => 1,
		},
	));
}

1;
__END__
